@@ -1,15 +1,17 @@
 use crate::annis::db::aql::{model::AnnotationComponentType, operators::RangeSpec};
+use crate::annis::errors::GraphAnnisError;
 use crate::annis::operator::{
     BinaryOperator, BinaryOperatorBase, BinaryOperatorIndex, BinaryOperatorSpec,
     EdgeAnnoSearchSpec, EstimationType,
 };
+use crate::errors::Result;
 use crate::graph::{GraphStatistic, GraphStorage, Match};
-use crate::AnnotationGraph;
+use crate::{try_as_boxed_iter, AnnotationGraph};
 use graphannis_core::{
-    annostorage::MatchGroup,
     graph::{ANNIS_NS, DEFAULT_ANNO_KEY, NODE_TYPE_KEY},
     types::{Component, Edge, NodeID},
 };
+use itertools::Itertools;
 use std::any::Any;
 use std::collections::{HashSet, VecDeque};
 use std::iter::FromIterator;
@@ -32,12 +34,15 @@ struct BaseEdgeOp {
 }
 
 impl BaseEdgeOp {
-    pub fn new(db: &AnnotationGraph, spec: BaseEdgeOpSpec) -> Option<BaseEdgeOp> {
+    pub fn new(db: &AnnotationGraph, spec: BaseEdgeOpSpec) -> Result<BaseEdgeOp> {
         let mut gs: Vec<Arc<dyn GraphStorage>> = Vec::new();
         for c in &spec.components {
-            gs.push(db.get_graphstorage(c)?);
+            let gs_for_component = db.get_graphstorage(c).ok_or_else(|| {
+                GraphAnnisError::ImpossibleSearch(format!("Component {} does not exist", &c))
+            })?;
+            gs.push(gs_for_component);
         }
-        Some(BaseEdgeOp {
+        Ok(BaseEdgeOp {
             gs,
             spec,
             max_nodes_estimate: db.get_node_annos().guess_max_count(
@@ -45,7 +50,7 @@ impl BaseEdgeOp {
                 &NODE_TYPE_KEY.name,
                 "node",
                 "node",
-            ),
+            )?,
             inverse: false,
         })
     }
@@ -59,7 +64,7 @@ impl BinaryOperatorSpec for BaseEdgeOpSpec {
         HashSet::from_iter(self.components.clone())
     }
 
-    fn create_operator<'a>(&self, db: &'a AnnotationGraph) -> Option<BinaryOperator<'a>> {
+    fn create_operator<'a>(&self, db: &'a AnnotationGraph) -> Result<BinaryOperator<'a>> {
         let optional_op = BaseEdgeOp::new(db, self.clone());
         optional_op.map(|op| BinaryOperator::Index(Box::new(op)))
     }
@@ -82,12 +87,12 @@ fn check_edge_annotation(
     gs: &dyn GraphStorage,
     source: NodeID,
     target: NodeID,
-) -> bool {
+) -> Result<bool> {
     match edge_anno {
         Some(EdgeAnnoSearchSpec::ExactValue { ns, name, val }) => {
             for a in gs
                 .get_anno_storage()
-                .get_annotations_for_item(&Edge { source, target })
+                .get_annotations_for_item(&Edge { source, target })?
             {
                 if name != &a.key.name {
                     continue;
@@ -103,14 +108,14 @@ fn check_edge_annotation(
                     }
                 }
                 // all checks passed, this edge has the correct annotation
-                return true;
+                return Ok(true);
             }
-            false
+            Ok(false)
         }
         Some(EdgeAnnoSearchSpec::NotExactValue { ns, name, val }) => {
             for a in gs
                 .get_anno_storage()
-                .get_annotations_for_item(&Edge { source, target })
+                .get_annotations_for_item(&Edge { source, target })?
             {
                 if name != &a.key.name {
                     continue;
@@ -125,9 +130,9 @@ fn check_edge_annotation(
                 }
 
                 // all checks passed, this edge has the correct annotation
-                return true;
+                return Ok(true);
             }
-            false
+            Ok(false)
         }
         Some(EdgeAnnoSearchSpec::RegexValue { ns, name, val }) => {
             let full_match_pattern = graphannis_core::util::regex_full_match(val);
@@ -135,7 +140,7 @@ fn check_edge_annotation(
             if let Ok(re) = re {
                 for a in gs
                     .get_anno_storage()
-                    .get_annotations_for_item(&Edge { source, target })
+                    .get_annotations_for_item(&Edge { source, target })?
                 {
                     if name != &a.key.name {
                         continue;
@@ -151,10 +156,10 @@ fn check_edge_annotation(
                     }
 
                     // all checks passed, this edge has the correct annotation
-                    return true;
+                    return Ok(true);
                 }
             }
-            false
+            Ok(false)
         }
         Some(EdgeAnnoSearchSpec::NotRegexValue { ns, name, val }) => {
             let full_match_pattern = graphannis_core::util::regex_full_match(val);
@@ -162,7 +167,7 @@ fn check_edge_annotation(
             if let Ok(re) = re {
                 for a in gs
                     .get_anno_storage()
-                    .get_annotations_for_item(&Edge { source, target })
+                    .get_annotations_for_item(&Edge { source, target })?
                 {
                     if name != &a.key.name {
                         continue;
@@ -178,12 +183,12 @@ fn check_edge_annotation(
                     }
 
                     // all checks passed, this edge has the correct annotation
-                    return true;
+                    return Ok(true);
                 }
             }
-            false
+            Ok(false)
         }
-        None => true,
+        None => Ok(true),
     }
 }
 
@@ -210,7 +215,7 @@ impl std::fmt::Display for BaseEdgeOp {
 }
 
 impl BinaryOperatorBase for BaseEdgeOp {
-    fn filter_match(&self, lhs: &Match, rhs: &Match) -> bool {
+    fn filter_match(&self, lhs: &Match, rhs: &Match) -> Result<bool> {
         for e in &self.gs {
             if self.inverse {
                 if e.is_connected(
@@ -218,42 +223,50 @@ impl BinaryOperatorBase for BaseEdgeOp {
                     lhs.node,
                     self.spec.dist.min_dist(),
                     self.spec.dist.max_dist(),
-                ) && check_edge_annotation(&self.spec.edge_anno, e.as_ref(), rhs.node, lhs.node)
-                {
-                    return true;
+                )? && check_edge_annotation(
+                    &self.spec.edge_anno,
+                    e.as_ref(),
+                    rhs.node,
+                    lhs.node,
+                )? {
+                    return Ok(true);
                 }
             } else if e.is_connected(
                 lhs.node,
                 rhs.node,
                 self.spec.dist.min_dist(),
                 self.spec.dist.max_dist(),
-            ) && check_edge_annotation(
+            )? && check_edge_annotation(
                 &self.spec.edge_anno,
                 e.as_ref(),
                 lhs.node,
                 rhs.node,
-            ) {
-                return true;
+            )? {
+                return Ok(true);
             }
         }
-        false
+        Ok(false)
     }
 
     fn is_reflexive(&self) -> bool {
         self.spec.is_reflexive
     }
 
-    fn get_inverse_operator<'a>(&self, _graph: &'a AnnotationGraph) -> Option<BinaryOperator<'a>> {
-        // Check if all graph storages have the same inverse cost.
-        // If not, we don't provide an inverse operator, because the plans would not account for the different costs
+    fn get_inverse_operator<'a>(
+        &self,
+        _graph: &'a AnnotationGraph,
+    ) -> Result<Option<BinaryOperator<'a>>> {
+        // Check if all graph storages have the same inverse cost. If not, we
+        // don't provide an inverse operator, because the plans would not
+        // account for the different costs
         for g in &self.gs {
             if !g.inverse_has_same_cost() {
-                return None;
+                return Ok(None);
             }
             if let Some(stat) = g.get_statistics() {
                 // If input and output estimations are too different, also don't provide a more costly inverse operator
                 if stat.inverse_fan_out_99_percentile > stat.fan_out_99_percentile {
-                    return None;
+                    return Ok(None);
                 }
             }
         }
@@ -263,13 +276,13 @@ impl BinaryOperatorBase for BaseEdgeOp {
             max_nodes_estimate: self.max_nodes_estimate,
             inverse: !self.inverse,
         };
-        Some(BinaryOperator::Index(Box::new(edge_op)))
+        Ok(Some(BinaryOperator::Index(Box::new(edge_op))))
     }
 
-    fn estimation_type(&self) -> EstimationType {
+    fn estimation_type(&self) -> Result<EstimationType> {
         if self.gs.is_empty() {
             // will not find anything
-            return EstimationType::Selectivity(0.0);
+            return Ok(EstimationType::Selectivity(0.0));
         }
 
         let max_nodes: f64 = self.max_nodes_estimate as f64;
@@ -285,7 +298,7 @@ impl BinaryOperatorBase for BaseEdgeOp {
                 let stats: &GraphStatistic = stats;
                 if stats.cyclic {
                     // can get all other nodes
-                    return EstimationType::Selectivity(1.0);
+                    return Ok(EstimationType::Selectivity(1.0));
                 }
                 // get number of nodes reachable from min to max distance
                 let max_dist = match self.spec.dist.max_dist() {
@@ -297,10 +310,13 @@ impl BinaryOperatorBase for BaseEdgeOp {
                 let min_path_length = std::cmp::max(0, self.spec.dist.min_dist() - 1) as i32;
 
                 if stats.avg_fan_out > 1.0 {
-                    // Assume two complete k-ary trees (with the average fan-out as k)
-                    // as defined in "Thomas Cormen: Introduction to algorithms (2009), page 1179)
-                    // with the maximum and minimum height. Calculate the number of nodes for both complete trees and
-                    // subtract them to get an estimation of the number of nodes that fullfull the path length criteria.
+                    // Assume two complete k-ary trees (with the average fan-out
+                    // as k) as defined in "Thomas Cormen: Introduction to
+                    // algorithms (2009), page 1179) with the maximum and
+                    // minimum height. Calculate the number of nodes for both
+                    // complete trees and subtract them to get an estimation of
+                    // the number of nodes that fullfull the path length
+                    // criteria.
                     let k = stats.avg_fan_out;
 
                     let reachable_max: f64 = ((k.powi(max_path_length) - 1.0) / (k - 1.0)).ceil();
@@ -310,8 +326,10 @@ impl BinaryOperatorBase for BaseEdgeOp {
 
                     gs_selectivity = reachable / max_nodes;
                 } else {
-                    // We can't use the formula for complete k-ary trees because we can't divide by zero and don't want negative
-                    // numbers. Use the simplified estimation with multiplication instead.
+                    // We can't use the formula for complete k-ary trees because
+                    // we can't divide by zero and don't want negative numbers.
+                    // Use the simplified estimation with multiplication
+                    // instead.
                     let reachable_max: f64 =
                         (stats.avg_fan_out * f64::from(max_path_length)).ceil();
                     let reachable_min: f64 =
@@ -326,19 +344,20 @@ impl BinaryOperatorBase for BaseEdgeOp {
             }
         } // end for
 
-        EstimationType::Selectivity(worst_sel)
+        Ok(EstimationType::Selectivity(worst_sel))
     }
 
-    fn edge_anno_selectivity(&self) -> Option<f64> {
+    fn edge_anno_selectivity(&self) -> Result<Option<f64>> {
         if let Some(ref edge_anno) = self.spec.edge_anno {
             let mut worst_sel = 0.0;
             for g in &self.gs {
                 let g: &Arc<dyn GraphStorage> = g;
                 let anno_storage = g.get_anno_storage();
-                let num_of_annos = anno_storage.number_of_annotations();
+                let num_of_annos = anno_storage.number_of_annotations()?;
                 if num_of_annos == 0 {
-                    // we won't be able to find anything if there are no annotations
-                    return Some(0.0);
+                    // we won't be able to find anything if there are no
+                    // annotations
+                    return Ok(Some(0.0));
                 } else {
                     let guessed_count = match edge_anno {
                         EdgeAnnoSearchSpec::ExactValue { val, ns, name } => {
@@ -348,40 +367,40 @@ impl BinaryOperatorBase for BaseEdgeOp {
                                     name,
                                     val,
                                     val,
-                                )
+                                )?
                             } else {
                                 anno_storage.number_of_annotations_by_name(
                                     ns.as_ref().map(String::as_str),
                                     name,
-                                )
+                                )?
                             }
                         }
                         EdgeAnnoSearchSpec::NotExactValue { val, ns, name } => {
                             let total = anno_storage.number_of_annotations_by_name(
                                 ns.as_ref().map(String::as_str),
                                 name,
-                            );
+                            )?;
                             total
                                 - anno_storage.guess_max_count(
                                     ns.as_ref().map(String::as_str),
                                     name,
                                     val,
                                     val,
-                                )
+                                )?
                         }
                         EdgeAnnoSearchSpec::RegexValue { val, ns, name } => anno_storage
-                            .guess_max_count_regex(ns.as_ref().map(String::as_str), name, val),
+                            .guess_max_count_regex(ns.as_ref().map(String::as_str), name, val)?,
                         EdgeAnnoSearchSpec::NotRegexValue { val, ns, name } => {
                             let total = anno_storage.number_of_annotations_by_name(
                                 ns.as_ref().map(String::as_str),
                                 name,
-                            );
+                            )?;
                             total
                                 - anno_storage.guess_max_count_regex(
                                     ns.as_ref().map(String::as_str),
                                     name,
                                     val,
-                                )
+                                )?
                         }
                     };
                     let g_sel: f64 = (guessed_count as f64) / (num_of_annos as f64);
@@ -390,59 +409,76 @@ impl BinaryOperatorBase for BaseEdgeOp {
                     }
                 }
             }
-            Some(worst_sel)
+            Ok(Some(worst_sel))
         } else {
-            Some(1.0)
+            Ok(Some(1.0))
         }
     }
 }
 
 impl BinaryOperatorIndex for BaseEdgeOp {
-    fn retrieve_matches(&self, lhs: &Match) -> Box<dyn Iterator<Item = Match>> {
+    fn retrieve_matches(&self, lhs: &Match) -> Box<dyn Iterator<Item = Result<Match>>> {
         let lhs = lhs.clone();
         let spec = self.spec.clone();
 
         if self.gs.len() == 1 {
             // directly return all matched nodes since when having only one component
             // no duplicates are possible
-            let result: VecDeque<Match> = if self.inverse {
+            let result: Result<VecDeque<_>> = if self.inverse {
                 self.gs[0]
                     .find_connected_inverse(lhs.node, spec.dist.min_dist(), spec.dist.max_dist())
                     .fuse()
-                    .filter(move |candidate| {
-                        check_edge_annotation(
+                    .map(move |candidate| {
+                        let candidate = candidate?;
+                        let has_annotation = check_edge_annotation(
                             &self.spec.edge_anno,
                             self.gs[0].as_ref(),
-                            *candidate,
-                            lhs.clone().node,
-                        )
+                            candidate,
+                            lhs.node,
+                        )?;
+                        Ok((candidate, has_annotation))
                     })
-                    .map(|n| Match {
-                        node: n,
-                        anno_key: DEFAULT_ANNO_KEY.clone(),
+                    .filter_map_ok(move |(n, has_annotation)| {
+                        if has_annotation {
+                            Some(Match {
+                                node: n,
+                                anno_key: DEFAULT_ANNO_KEY.clone(),
+                            })
+                        } else {
+                            None
+                        }
                     })
                     .collect()
             } else {
                 self.gs[0]
                     .find_connected(lhs.node, spec.dist.min_dist(), spec.dist.max_dist())
                     .fuse()
-                    .filter(move |candidate| {
-                        check_edge_annotation(
+                    .map(move |candidate| {
+                        let candidate = candidate?;
+                        let has_annotation = check_edge_annotation(
                             &self.spec.edge_anno,
                             self.gs[0].as_ref(),
-                            lhs.clone().node,
-                            *candidate,
-                        )
+                            lhs.node,
+                            candidate,
+                        )?;
+                        Ok((candidate, has_annotation))
                     })
-                    .map(|n| Match {
-                        node: n,
-                        anno_key: DEFAULT_ANNO_KEY.clone(),
+                    .filter_map_ok(move |(n, has_annotation)| {
+                        if has_annotation {
+                            Some(Match {
+                                node: n,
+                                anno_key: DEFAULT_ANNO_KEY.clone(),
+                            })
+                        } else {
+                            None
+                        }
                     })
                     .collect()
             };
-            Box::new(result.into_iter())
+            let result = try_as_boxed_iter!(result);
+            Box::new(result.into_iter().map(Ok))
         } else {
-            let mut all: MatchGroup = if self.inverse {
+            let all: Result<Vec<_>> = if self.inverse {
                 self.gs
                     .iter()
                     .flat_map(move |e| {
@@ -455,17 +491,25 @@ impl BinaryOperatorIndex for BaseEdgeOp {
                                 spec.dist.max_dist(),
                             )
                             .fuse()
-                            .filter(move |candidate| {
-                                check_edge_annotation(
+                            .map(move |candidate| {
+                                let candidate = candidate?;
+                                let has_annotation = check_edge_annotation(
                                     &self.spec.edge_anno,
                                     e.as_ref(),
-                                    *candidate,
-                                    lhs.clone().node,
-                                )
+                                    candidate,
+                                    lhs.node,
+                                )?;
+                                Ok((candidate, has_annotation))
                             })
-                            .map(|n| Match {
-                                node: n,
-                                anno_key: DEFAULT_ANNO_KEY.clone(),
+                            .filter_map_ok(move |(n, has_annotation)| {
+                                if has_annotation {
+                                    Some(Match {
+                                        node: n,
+                                        anno_key: DEFAULT_ANNO_KEY.clone(),
+                                    })
+                                } else {
+                                    None
+                                }
                             })
                     })
                     .collect()
@@ -478,24 +522,33 @@ impl BinaryOperatorIndex for BaseEdgeOp {
                         e.as_ref()
                             .find_connected(lhs.node, spec.dist.min_dist(), spec.dist.max_dist())
                             .fuse()
-                            .filter(move |candidate| {
-                                check_edge_annotation(
+                            .map(move |candidate| {
+                                let candidate = candidate?;
+                                let has_annotation = check_edge_annotation(
                                     &self.spec.edge_anno,
                                     e.as_ref(),
-                                    lhs.clone().node,
-                                    *candidate,
-                                )
+                                    lhs.node,
+                                    candidate,
+                                )?;
+                                Ok((candidate, has_annotation))
                             })
-                            .map(|n| Match {
-                                node: n,
-                                anno_key: DEFAULT_ANNO_KEY.clone(),
+                            .filter_map_ok(move |(n, has_annotation)| {
+                                if has_annotation {
+                                    Some(Match {
+                                        node: n,
+                                        anno_key: DEFAULT_ANNO_KEY.clone(),
+                                    })
+                                } else {
+                                    None
+                                }
                             })
                     })
                     .collect()
             };
+            let mut all = try_as_boxed_iter!(all);
             all.sort_unstable();
             all.dedup();
-            Box::new(all.into_iter())
+            Box::new(all.into_iter().map(Ok))
         }
     }
 
@@ -521,7 +574,7 @@ impl BinaryOperatorSpec for DominanceSpec {
         )
     }
 
-    fn create_operator<'a>(&self, db: &'a AnnotationGraph) -> Option<BinaryOperator<'a>> {
+    fn create_operator<'a>(&self, db: &'a AnnotationGraph) -> Result<BinaryOperator<'a>> {
         let components =
             db.get_all_components(Some(AnnotationComponentType::Dominance), Some(&self.name));
         let op_str = if self.name.is_empty() {
@@ -565,7 +618,7 @@ impl BinaryOperatorSpec for PointingSpec {
         )
     }
 
-    fn create_operator<'a>(&self, db: &'a AnnotationGraph) -> Option<BinaryOperator<'a>> {
+    fn create_operator<'a>(&self, db: &'a AnnotationGraph) -> Result<BinaryOperator<'a>> {
         let components =
             db.get_all_components(Some(AnnotationComponentType::Pointing), Some(&self.name));
         let op_str = if self.name.is_empty() {
@@ -612,7 +665,7 @@ impl BinaryOperatorSpec for PartOfSubCorpusSpec {
         components
     }
 
-    fn create_operator<'a>(&self, db: &'a AnnotationGraph) -> Option<BinaryOperator<'a>> {
+    fn create_operator<'a>(&self, db: &'a AnnotationGraph) -> Result<BinaryOperator<'a>> {
         let components = vec![Component::new(
             AnnotationComponentType::PartOf,
             ANNIS_NS.into(),
