@@ -1,5 +1,5 @@
 use super::aql::model::{AnnotationComponentType, TOK_WHITESPACE_AFTER, TOK_WHITESPACE_BEFORE};
-use crate::annis::db::corpusstorage::SALT_URI_ENCODE_SET;
+use crate::annis::db::corpusstorage::NODE_NAME_ENCODE_SET;
 use crate::annis::errors::*;
 use crate::annis::util::create_str_vec_key;
 use crate::update::{GraphUpdate, UpdateEvent};
@@ -491,6 +491,9 @@ where
 
     let mut rules_by_order: Vec<(i64, bool, VisualizerRule)> = DEFAULT_VISUALIZER_RULES.clone();
 
+    let order_col = if is_annis_33 { 7 } else { 6 };
+    let mapping_col = if is_annis_33 { 8 } else { 7 };
+
     for result in resolver_tab_csv.records() {
         let line = result?;
 
@@ -506,11 +509,13 @@ where
 
         let visibility = get_field_not_null(&line, 6, "visibility", &resolver_tab_path)?;
 
-        let order = get_field(&line, 7, "order", &resolver_tab_path)?
+        let order = get_field(&line, order_col, "order", &resolver_tab_path)?
             .map(|order| order.parse::<i64>().unwrap_or_default())
             .unwrap_or_default();
         let mappings: BTreeMap<std::string::String, std::string::String> =
-            if let Ok(mappings_field) = get_field(&line, 8, "mappings", &resolver_tab_path) {
+            if let Ok(mappings_field) =
+                get_field(&line, mapping_col, "mappings", &resolver_tab_path)
+            {
                 mappings_field
                     .unwrap_or_default()
                     .split(';')
@@ -900,7 +905,7 @@ where
         let pre_order = get_field_not_null(&line, 4, "pre", &corpus_tab_path)?.parse::<u32>()?;
         let post_order = get_field_not_null(&line, 5, "post", &corpus_tab_path)?.parse::<u32>()?;
 
-        let normalized_name = utf8_percent_encode(&name, SALT_URI_ENCODE_SET);
+        let normalized_name = utf8_percent_encode(&name, NODE_NAME_ENCODE_SET);
 
         corpus_by_id.insert(
             id,
@@ -2137,7 +2142,7 @@ fn add_subcorpora(
         let (text_key, text) = text?;
         // add text node (including its name)
         if let Some(corpus_ref) = text_key.corpus_ref {
-            let text_name = utf8_percent_encode(&text.name, SALT_URI_ENCODE_SET).to_string();
+            let text_name = utf8_percent_encode(&text.name, NODE_NAME_ENCODE_SET).to_string();
             let subcorpus_full_name = get_corpus_path(corpus_ref, corpus_table)?;
             let text_full_name = format!("{}#{}", &subcorpus_full_name, &text_name);
 
@@ -2238,7 +2243,7 @@ mod tests {
             pre: 0,
             post: 3,
             name: "corpus".into(),
-            normalized_name: "document".into(),
+            normalized_name: "corpus".into(),
         };
         corpus_by_id.insert(0, document);
         corpus_by_id.insert(1, corpus);
